///////////////////////////////////////////////////////////////////////////////
//                                                                           //
//  Notice to licensees:                                                     //
//                                                                           //
//  This source code is the exclusive, proprietary intellectual property of  //
//  Sharkysoft (sharkysoft.com).  You may view this source code as a         //
//  supplement to other product documentation, but you may not distribute    //
//  it or use it for any other purpose without written consent from          //
//  Sharkysoft.                                                              //
//                                                                           //
//  You are permitted to modify and recompile this source code, but you may  //
//  not remove this notice.  If you add features to or fix errors in this    //
//  code, please consider sharing your changes with Sharkysoft for possible  //
//  incorporation into future releases of the product.  Thanks!              //
//                                                                           //
//  For more information about Sharkysoft products and services, please      //
//  visit Sharkysoft on the web at                                           //
//                                                                           //
//       http://sharkysoft.com/                                              //
//                                                                           //
//  Thank you for using Lava!                                                //
//                                                                           //
///////////////////////////////////////////////////////////////////////////////



package lava.array;



/******************************************************************************
Splits/concatenates primitive integer types using "little endian" byte ordering.

<p><b>Details:</b>  The functions in this class perform the same tasks as the functions in {@link BigEndianArray <code>BigEndianArray</code>}, except that "little endian" byte ordering is used instead of "big endian" byte ordering.</p>

@see BigEndianArray
@since a long time ago
@version 2000.05.26
@author Sharky
******************************************************************************/

public final class LittleEndianArray
{



	public final static char[] concat (byte[] src, int from, char[] dest, int to, int amount)
	{
		if (dest == null)
			dest = new char [to + amount / 2];
		for (int i = 0; i < amount; i += 2)
		{
			dest [to ++] = (char)
			(
				(src [from + 0] & 0xff) << 0
			|	(src [from + 1] & 0xff) << 8
			);
			from += 2;
		}
		return dest;
	}



	public final static short[] concat (byte[] src, int from, short[] dest, int to, int amount)
	{
		if (dest == null)
			dest = new short [to + amount / 2];
		for (int i = 0; i < amount; i += 2)
		{
			dest [to ++] = (short)
			(
				(src [from + 0] & 0xff) << 0
			|	(src [from + 1] & 0xff) << 8
			);
			from += 2;
		}
		return dest;
	}



	public final static int[] concat (byte[] src, int from, int[] dest, int to, int amount)
	{
		if (dest == null)
			dest = new int [to + amount / 4];
		for (int i = 0; i < amount; i += 4)
		{
			dest [to ++] = (int)
			(
				(src [from + 0] & 0xff) << 0
			|	(src [from + 1] & 0xff) << 8
			|	(src [from + 2] & 0xff) << 16
			|	(src [from + 3] & 0xff) << 24
			);
			from += 4;
		}
		return dest;
	}



	public final static long[] concat (byte[] src, int from, long[] dest, int to, int amount)
	{
		if (dest == null)
			dest = new long [to + amount / 8];
		for (int i = 0; i < amount; i += 8)
		{
			dest [to ++] = (long)
			(
				(src [from + 0] & 0xffL) << 0
			|	(src [from + 1] & 0xffL) << 8
			|	(src [from + 2] & 0xffL) << 16
			|	(src [from + 3] & 0xffL) << 24
			|	(src [from + 4] & 0xffL) << 32
			|	(src [from + 5] & 0xffL) << 40
			|	(src [from + 6] & 0xffL) << 48
			|	(src [from + 7] & 0xffL) << 56
			);
			from += 8;
		}
		return dest;
	}



	public final static byte[] split (char[] src, int from, byte[] dest, int to, int amount)
	{
		if (dest == null)
			dest = new byte [to + amount * 2];
		for (int i = 0; i < amount; ++ i)
		{
			dest [to ++] = (byte) (src [from] >> 0);
			dest [to ++] = (byte) (src [from] >> 8);
			++ from;
		}
		return dest;
	}



	public final static int[] concat (char[] src, int from, int[] dest, int to, int amount)
	{
		if (dest == null)
			dest = new int [to + amount / 2];
		for (int i = 0; i < amount; i += 2)
		{
			dest [to ++] = (int)
			(
				(src [from + 0] & 0xffff) << 0
			|	(src [from + 1] & 0xffff) << 16
			);
			from += 2;
		}
		return dest;
	}



	public final static long[] concat (char[] src, int from, long[] dest, int to, int amount)
	{
		if (dest == null)
			dest = new long [to + amount / 4];
		for (int i = 0; i < amount; i += 4)
		{
			dest [to ++] = (long)
			(
				(src [from + 0] & 0xffffL) << 0
			|	(src [from + 1] & 0xffffL) << 16
			|	(src [from + 2] & 0xffffL) << 32
			|	(src [from + 3] & 0xffffL) << 48
			);
			from += 4;
		}
		return dest;
	}



	public final static byte[] split (short[] src, int from, byte[] dest, int to, int amount)
	{
		if (dest == null)
			dest = new byte [to + amount * 2];
		for (int i = 0; i < amount; ++ i)
		{
			dest [to ++] = (byte) (src [from] >> 0);
			dest [to ++] = (byte) (src [from] >> 8);
			++ from;
		}
		return dest;
	}



	public final static int[] concat (short[] src, int from, int[] dest, int to, int amount)
	{
		if (dest == null)
			dest = new int [to + amount / 2];
		for (int i = 0; i < amount; i += 2)
		{
			dest [to ++] = (int)
			(
				(src [from + 0] & 0xffff) << 0
			|	(src [from + 1] & 0xffff) << 16
			);
			from += 2;
		}
		return dest;
	}



	public final static long[] concat (short[] src, int from, long[] dest, int to, int amount)
	{
		if (dest == null)
			dest = new long [to + amount / 4];
		for (int i = 0; i < amount; i += 4)
		{
			dest [to ++] = (long)
			(
				(src [from + 0] & 0xffffL) << 0
			|	(src [from + 1] & 0xffffL) << 16
			|	(src [from + 2] & 0xffffL) << 32
			|	(src [from + 3] & 0xffffL) << 48
			);
			from += 4;
		}
		return dest;
	}



	public final static byte[] split (int[] src, int from, byte[] dest, int to, int amount)
	{
		if (dest == null)
			dest = new byte [to + amount * 4];
		for (int i = 0; i < amount; ++ i)
		{
			dest [to ++] = (byte) (src [from] >> 0);
			dest [to ++] = (byte) (src [from] >> 8);
			dest [to ++] = (byte) (src [from] >> 16);
			dest [to ++] = (byte) (src [from] >> 24);
			++ from;
		}
		return dest;
	}



	public final static char[] split (int[] src, int from, char[] dest, int to, int amount)
	{
		if (dest == null)
			dest = new char [to + amount * 2];
		for (int i = 0; i < amount; ++ i)
		{
			dest [to ++] = (char) (src [from] >> 0);
			dest [to ++] = (char) (src [from] >> 16);
			++ from;
		}
		return dest;
	}



	public final static short[] split (int[] src, int from, short[] dest, int to, int amount)
	{
		if (dest == null)
			dest = new short [to + amount * 2];
		for (int i = 0; i < amount; ++ i)
		{
			dest [to ++] = (short) (src [from] >> 0);
			dest [to ++] = (short) (src [from] >> 16);
			++ from;
		}
		return dest;
	}



	public final static long[] concat (int[] src, int from, long[] dest, int to, int amount)
	{
		if (dest == null)
			dest = new long [to + amount / 2];
		for (int i = 0; i < amount; i += 2)
		{
			dest [to ++] = (long)
			(
				(src [from + 0] & 0xffffffffL) << 0
			|	(src [from + 1] & 0xffffffffL) << 32
			);
			from += 2;
		}
		return dest;
	}



	public final static byte[] split (long[] src, int from, byte[] dest, int to, int amount)
	{
		if (dest == null)
			dest = new byte [to + amount * 8];
		for (int i = 0; i < amount; ++ i)
		{
			dest [to ++] = (byte) (src [from] >> 0);
			dest [to ++] = (byte) (src [from] >> 8);
			dest [to ++] = (byte) (src [from] >> 16);
			dest [to ++] = (byte) (src [from] >> 24);
			dest [to ++] = (byte) (src [from] >> 32);
			dest [to ++] = (byte) (src [from] >> 40);
			dest [to ++] = (byte) (src [from] >> 48);
			dest [to ++] = (byte) (src [from] >> 56);
			++ from;
		}
		return dest;
	}



	public final static char[] split (long[] src, int from, char[] dest, int to, int amount)
	{
		if (dest == null)
			dest = new char [to + amount * 4];
		for (int i = 0; i < amount; ++ i)
		{
			dest [to ++] = (char) (src [from] >> 0);
			dest [to ++] = (char) (src [from] >> 16);
			dest [to ++] = (char) (src [from] >> 32);
			dest [to ++] = (char) (src [from] >> 48);
			++ from;
		}
		return dest;
	}



	public final static short[] split (long[] src, int from, short[] dest, int to, int amount)
	{
		if (dest == null)
			dest = new short [to + amount * 4];
		for (int i = 0; i < amount; ++ i)
		{
			dest [to ++] = (short) (src [from] >> 0);
			dest [to ++] = (short) (src [from] >> 16);
			dest [to ++] = (short) (src [from] >> 32);
			dest [to ++] = (short) (src [from] >> 48);
			++ from;
		}
		return dest;
	}



	public final static int[] split (long[] src, int from, int[] dest, int to, int amount)
	{
		if (dest == null)
			dest = new int [to + amount * 2];
		for (int i = 0; i < amount; ++ i)
		{
			dest [to ++] = (int) (src [from] >> 0);
			dest [to ++] = (int) (src [from] >> 32);
			++ from;
		}
		return dest;
	}



}



