///////////////////////////////////////////////////////////////////////////////
//                                                                           //
//  Notice to licensees:                                                     //
//                                                                           //
//  This source code is the exclusive, proprietary intellectual property of  //
//  Sharkysoft (sharkysoft.com).  You may view this source code as a         //
//  supplement to other product documentation, but you may not distribute    //
//  it or use it for any other purpose without written consent from          //
//  Sharkysoft.                                                              //
//                                                                           //
//  You are permitted to modify and recompile this source code, but you may  //
//  not remove this notice.  If you add features to or fix errors in this    //
//  code, please consider sharing your changes with Sharkysoft for possible  //
//  incorporation into future releases of the product.  Thanks!              //
//                                                                           //
//  For more information about Sharkysoft products and services, please      //
//  visit Sharkysoft on the web at                                           //
//                                                                           //
//       http://sharkysoft.com/                                              //
//                                                                           //
//  Thank you for using Lava!                                                //
//                                                                           //
///////////////////////////////////////////////////////////////////////////////



package lava.array;



import lava.konst.BYTE;
import lava.konst.INT;
import lava.konst.SHORT;



/******************************************************************************
Casts arrays of signed primitives.

<p><b>Details:</b>  <code>SignedArrayCaster</code> is a collection of functions that cast arrays of signed primitives from one type to another using standard <em>Lava</em> primitive widening and narrowing conventions.  In particular, alternate provisions for some of the less intuitive Java widening and narrowing conventions are presented here.  For casting operations involving <em>unsigned</em> types, use <a href=UnsignedArrayCaster.html><code>UnsignedArrayCaster</code></a>.  For casting operations where the default Java casting behavior is acceptable, use <a href=ArrayCaster.html><code>ArrayCaster</code></a>.</p>

<p>The signatures for functions in this class follow the same rules as described in {@link ArrayCaster <code>ArrayCaster</code>}.</p>

@see ArrayCaster
@see UnsignedArrayCaster

@author Sharky
@version 1999.09.11
******************************************************************************/

public abstract class SignedArrayCaster
	extends ArrayCaster
{



	public static short[] toShorts
	(
		double[] src,
		int from,
		short[] dest,
		int to,
		int amt
	)
	{
		if (amt < 0)
			throw new IllegalArgumentException ("amt==" + amt);
		if (src == null)
			return null;
		if (dest == null)
			dest = new short [to + amt];
		for (int i = 0; i < amt; ++ i)
		{
			int d = (int) src [from + i];
			if (d > INT.MAX_SSHORT)
				dest [to + i] = SHORT.MAX_SSHORT;
			else if (d < INT.MIN_SSHORT)
				dest [to + i] = SHORT.MIN_SSHORT;
			else
				dest [to + i] = (short) d;
		}
		return dest;
	}



	public static short[] toShorts
	(
		float[] src,
		int from,
		short[] dest,
		int to,
		int amt
	)
	{
		if (amt < 0)
			throw new IllegalArgumentException ("amt==" + amt);
		if (src == null)
			return null;
		if (dest == null)
			dest = new short [to + amt];
		for (int i = 0; i < amt; ++ i)
		{
			int d = (int) src [from + i];
			if (d > INT.MAX_SSHORT)
				dest [to + i] = SHORT.MAX_SSHORT;
			else if (d < INT.MIN_SSHORT)
				dest [to + i] = SHORT.MIN_SSHORT;
			else
				dest [to + i] = (short) d;
		}
		return dest;
	}



	public static byte[] toBytes
	(
		double[] src,
		int from,
		byte[] dest,
		int to,
		int amt
	)
	{
		if (amt < 0)
			throw new IllegalArgumentException ("amt==" + amt);
		if (src == null)
			return null;
		if (dest == null)
			dest = new byte [to + amt];
		for (int i = 0; i < amt; ++ i)
		{
			int d = (int) src [from + i];
			if (d > INT.MAX_SBYTE)
				dest [to + i] = BYTE.MAX_SBYTE;
			else if (d < INT.MIN_SBYTE)
				dest [to + i] = BYTE.MIN_SBYTE;
			else
				dest [to + i] = (byte) d;
		}
		return dest;
	}



	public static byte[] toBytes
	(
		float[] src,
		int from,
		byte[] dest,
		int to,
		int amt
	)
	{
		if (amt < 0)
			throw new IllegalArgumentException ("amt==" + amt);
		if (src == null)
			return null;
		if (dest == null)
			dest = new byte [to + amt];
		for (int i = 0; i < amt; ++ i)
		{
			int d = (int) src [from + i];
			if (d > INT.MAX_SBYTE)
				dest [to + i] = BYTE.MAX_SBYTE;
			else if (d < INT.MIN_SBYTE)
				dest [to + i] = BYTE.MIN_SBYTE;
			else
				dest [to + i] = (byte) d;
		}
		return dest;
	}



}



