///////////////////////////////////////////////////////////////////////////////
//                                                                           //
//  Notice to licensees:                                                     //
//                                                                           //
//  This source code is the exclusive, proprietary intellectual property of  //
//  Sharkysoft (sharkysoft.com).  You may view this source code as a         //
//  supplement to other product documentation, but you may not distribute    //
//  it or use it for any other purpose without written consent from          //
//  Sharkysoft.                                                              //
//                                                                           //
//  You are permitted to modify and recompile this source code, but you may  //
//  not remove this notice.  If you add features to or fix errors in this    //
//  code, please consider sharing your changes with Sharkysoft for possible  //
//  incorporation into future releases of the product.  Thanks!              //
//                                                                           //
//  For more information about Sharkysoft products and services, please      //
//  visit Sharkysoft on the web at                                           //
//                                                                           //
//       http://sharkysoft.com/                                              //
//                                                                           //
//  Thank you for using Lava!                                                //
//                                                                           //
///////////////////////////////////////////////////////////////////////////////



package lava.math;



/******************************************************************************
Miscellaneous mathematical operations.

<p><b>Details:</b> <code>MathToolbox</code> is a collection of static methods for performing mathematical operations.</p>

<!--SHOWSOURCE-->

@version 1998.10.26
@since 1998
******************************************************************************/

public class MathToolbox
{



	/**********************************************************************
	Arbitrary modulus operation.

	<p><b>Details:</b>  The remainder operator <code>%</code> is useful for performing modulus operations, but only if the range begins at 0 and the domain is non-negative.  This method provides a more generic modulus operation because it allows negative domains and arbitrarily positioned ranges.</p>

	<p><code>lowerbound</code> is the beginning of the range, and <code>upperbound</code> is one more than the last value allowed in the range.  (Obviously, <code>upperbound - lowerbound</code> is the period of modulation.)  <code>value</code> is the value to modulate.</p>

	<p>The returned value, <var>retval</var>, is such that:</p>

	<ul>
	 <li><code>lowerbound</code> <= <var>retval</var>
	 <li><var>retval</var> < <code>upperbound</code>
	 <li><code>(value - </code><var>retval</var><code>)</code> is a multiple of <code>(upperbound - lowerbound)</code>
	</ul>

	<p>The % operator has these properties, too, but only for lowerbound == 0 and value >= 0.</p>

	@param value value to modulate
	@param lowerbound bottom of range
	@param upperbound top of range
	@return modulated value within [lowerbound..upperbound)

	@version 1998.10.31
	@since 1988.10.31
	**********************************************************************/

	public final static int mod (int value, int lowerbound, int upperbound)
	{
		int period = upperbound - lowerbound;
		if (value < lowerbound)
			value += (lowerbound - value + period - 1) / period * period;
		else if (value >= upperbound)
			value -= (value - upperbound + period) / period * period;
		return value;
	}



}



