///////////////////////////////////////////////////////////////////////////////
//                                                                           //
//  Notice to licensees:                                                     //
//                                                                           //
//  This source code is the exclusive, proprietary intellectual property of  //
//  Sharkysoft (sharkysoft.com).  You may view this source code as a         //
//  supplement to other product documentation, but you may not distribute    //
//  it or use it for any other purpose without written consent from          //
//  Sharkysoft.                                                              //
//                                                                           //
//  You are permitted to modify and recompile this source code, but you may  //
//  not remove this notice.  If you add features to or fix errors in this    //
//  code, please consider sharing your changes with Sharkysoft for possible  //
//  incorporation into future releases of the product.  Thanks!              //
//                                                                           //
//  For more information about Sharkysoft products and services, please      //
//  visit Sharkysoft on the web at                                           //
//                                                                           //
//       http://sharkysoft.com/                                              //
//                                                                           //
//  Thank you for using Lava!                                                //
//                                                                           //
///////////////////////////////////////////////////////////////////////////////



package lava.security;



/******************************************************************************
Defines methods for block ciphers.

<p><b>Details:</b>  An <code>IBlockCipher64</code> is a fixed-width block cipher.  This interface exists primarily to support the interchangability of <code>DesEngine</code> and <code>TripleDesEngine</code>, both of which are 64-bit block ciphers, but it may be used to represent other 64-bit block ciphers as well.</p>

<p><code>IBlockCipher64</code> defines two methods for processing blocks of data: <code>encrypt</code> and <code>decrypt</code>.  In both methods, there is a source array and a destination array.  64-bit blocks are read from the source array, processed, and then written into the destination array as 64-bit blocks.  The two arrays may be different types, and need not necessarily be <code>long</code> arrays, but symbols will always be consumed and produced in 64-bit blocks.  If the source and destination arrays are the same object, then the input symbols will be overwritten with output symbols.</p>

<p>Both source and destination arrays must be arrays of primitive integer types (<code>byte</code>, <code>short</code>, <code>int</code>, or <code>long</code>).  If <code>byte</code>s are used, then they are processed 8 at a time; if <code>short</code>s are used, then they are processed 4 at a time; and if <code>int</code>s are used, then they are processed in pairs.  When <code>byte</code>s, <code>short</code>s, or <code>int</code>s are used to represent 64-bit blocks, they are arranged in big endian order.</p>

<p>The <var>length</var> parameter for methods in this interface does <em>not</em> count 64-bit blocks, but rather the number of array elements in the source array to process, in multiples that define whole blocks.  For example, if two 64-bit blocks are to be read from an int array, then the <var>length</var> parameter should be set to 4.</p>

<form>
<p>Clip-source:<br>
<textarea rows=10 cols=80 onChange="this.form.reset();">
*	public void encrypt (Object src, Object dest, int length)
*	{
*		throw new lava.NotImplementedException ();
*	}
*
*
*
*	public void decrypt (Object src, Object dest, int length)
*	{
*		throw new lava.NotImplementedException ();
*	}
*
*
*
</textarea></form>

@since 1999.08.08
@version 1999.08.18
******************************************************************************/

public interface IBlockCipher64
{



	/**********************************************************************
	Encrypts blocks.

	<p><b>Details:</b>  This method encrypts a vector of blocks.  Refer to the interface introduction for details.</p>

	@param src the source array
	@param dest the destination array
	@param length number of elements in src to process
	**********************************************************************/

	void encrypt (Object src, Object dest, int length);



	/**********************************************************************
	Decrypts blocks.

	<p><b>Details:</b>  This method decrypts a vector of blocks.  Refer to the interface introduction for details.</p>

	@param src the source array
	@param dest the destination array
	@param length number of elements in src to process
	**********************************************************************/

	void decrypt (Object src, Object dest, int length);



}



