///////////////////////////////////////////////////////////////////////////////
//                                                                           //
//  Notice to licensees:                                                     //
//                                                                           //
//  This source code is the exclusive, proprietary intellectual property of  //
//  Sharkysoft (sharkysoft.com).  You may view this source code as a         //
//  supplement to other product documentation, but you may not distribute    //
//  it or use it for any other purpose without written consent from          //
//  Sharkysoft.                                                              //
//                                                                           //
//  You are permitted to modify and recompile this source code, but you may  //
//  not remove this notice.  If you add features to or fix errors in this    //
//  code, please consider sharing your changes with Sharkysoft for possible  //
//  incorporation into future releases of the product.  Thanks!              //
//                                                                           //
//  For more information about Sharkysoft products and services, please      //
//  visit Sharkysoft on the web at                                           //
//                                                                           //
//       http://sharkysoft.com/                                              //
//                                                                           //
//  Thank you for using Lava!                                                //
//                                                                           //
///////////////////////////////////////////////////////////////////////////////



package lava.security.des;



import lava.security.IBlockCipher64;



/******************************************************************************
Decrypts using DES-ECB.

<p><b>Details:</b> <code>DesEcbDecoder</code> decrypts <code>long</code> arrays using DES (or 3DES) in ECB mode.  <strong>Please refer to the superclass documentation for important information regarding usage.</strong></p>

@since 2001.02.02
@author Sharky
******************************************************************************/

public class DesEcbDecoder extends DesEcbCoder
{



	/**********************************************************************
	Sets decryption engine.

	<p><b>Details:</b>  This constructor sets the 64-bit block decryption engine (<var>engine</var>) used for decryption.  <var>engine</var> can be an instance of <code>DesEngine</code>, <code>TripleDesEngine</code>, or any other 64-bit block cipher that satisfies the <code>IBlockCipher64</code> interface contract.</p>

	@param engine the engine
	**********************************************************************/

	public DesEcbDecoder (IBlockCipher64 engine)
	{
		super (engine);
	}



	/**********************************************************************
	Decrypts longs.

	<p><b>Details:</b>  <code>continueTransform</code> decrypts the plain text stored in <var>src</var> and stores the resulting cipher text in the provided output array (<var>dest</var>).  Both arrays must be <code>long</code> arrays.  If <var>dest</var> is too short to store the result, the overflow is discarded.</p>

	@param src cipher text
	@param dest plain text
	@param length number of cipher text symbols
	@return number of plain text symbols stored or discarded

	@version 2000.03.06
	**********************************************************************/

	public int continueTransform (Object src, Object dest, int length)
	{
		engine . decrypt ((long[]) src, (long[]) dest, length);
		return length;
	}



}



