///////////////////////////////////////////////////////////////////////////////
//                                                                           //
//  Notice to licensees:                                                     //
//                                                                           //
//  This source code is the exclusive, proprietary intellectual property of  //
//  Sharkysoft (sharkysoft.com).  You may view this source code as a         //
//  supplement to other product documentation, but you may not distribute    //
//  it or use it for any other purpose without written consent from          //
//  Sharkysoft.                                                              //
//                                                                           //
//  You are permitted to modify and recompile this source code, but you may  //
//  not remove this notice.  If you add features to or fix errors in this    //
//  code, please consider sharing your changes with Sharkysoft for possible  //
//  incorporation into future releases of the product.  Thanks!              //
//                                                                           //
//  For more information about Sharkysoft products and services, please      //
//  visit Sharkysoft on the web at                                           //
//                                                                           //
//       http://sharkysoft.com/                                              //
//                                                                           //
//  Thank you for using Lava!                                                //
//                                                                           //
///////////////////////////////////////////////////////////////////////////////



package lava.security.des;



/******************************************************************************
Collapsed S-P representation.

<p><b>Details:</b>  This class contains an optimized representation of the S and P box functions.  The 6 bit values that usually select S box values are direct indeces into the table.  The indexed values are already P-permuted, so that the indexed values can be immediately ORed together to obtain the result of the mixing function ("<i>f</i>" in the standard).</p>

@since 1999.08.04
@author Sharky
******************************************************************************/

final class DesSPBoxes
{



	/**********************************************************************
	Reordered, pre-shifted, pre-permuted values.
	**********************************************************************/

	static final int[][] values =
	{
		{
			0x08000820, 0x00000800, 0x00020000, 0x08020820,
			0x08000000, 0x08000820, 0x00000020, 0x08000000,
			0x00020020, 0x08020000, 0x08020820, 0x00020800,
			0x08020800, 0x00020820, 0x00000800, 0x00000020,
			0x08020000, 0x08000020, 0x08000800, 0x00000820,
			0x00020800, 0x00020020, 0x08020020, 0x08020800,
			0x00000820, 0x00000000, 0x00000000, 0x08020020,
			0x08000020, 0x08000800, 0x00020820, 0x00020000,
			0x00020820, 0x00020000, 0x08020800, 0x00000800,
			0x00000020, 0x08020020, 0x00000800, 0x00020820,
			0x08000800, 0x00000020, 0x08000020, 0x08020000,
			0x08020020, 0x08000000, 0x00020000, 0x08000820,
			0x00000000, 0x08020820, 0x00020020, 0x08000020,
			0x08020000, 0x08000800, 0x08000820, 0x00000000,
			0x08020820, 0x00020800, 0x00020800, 0x00000820,
			0x00000820, 0x00020020, 0x08000000, 0x08020800
		},
		{
			0x00100000, 0x02100001, 0x02000401, 0x00000000,
			0x00000400, 0x02000401, 0x00100401, 0x02100400,
			0x02100401, 0x00100000, 0x00000000, 0x02000001,
			0x00000001, 0x02000000, 0x02100001, 0x00000401,
			0x02000400, 0x00100401, 0x00100001, 0x02000400,
			0x02000001, 0x02100000, 0x02100400, 0x00100001,
			0x02100000, 0x00000400, 0x00000401, 0x02100401,
			0x00100400, 0x00000001, 0x02000000, 0x00100400,
			0x02000000, 0x00100400, 0x00100000, 0x02000401,
			0x02000401, 0x02100001, 0x02100001, 0x00000001,
			0x00100001, 0x02000000, 0x02000400, 0x00100000,
			0x02100400, 0x00000401, 0x00100401, 0x02100400,
			0x00000401, 0x02000001, 0x02100401, 0x02100000,
			0x00100400, 0x00000000, 0x00000001, 0x02100401,
			0x00000000, 0x00100401, 0x02100000, 0x00000400,
			0x02000001, 0x02000400, 0x00000400, 0x00100001
		},
		{
			0x10000008, 0x10200000, 0x00002000, 0x10202008,
			0x10200000, 0x00000008, 0x10202008, 0x00200000,
			0x10002000, 0x00202008, 0x00200000, 0x10000008,
			0x00200008, 0x10002000, 0x10000000, 0x00002008,
			0x00000000, 0x00200008, 0x10002008, 0x00002000,
			0x00202000, 0x10002008, 0x00000008, 0x10200008,
			0x10200008, 0x00000000, 0x00202008, 0x10202000,
			0x00002008, 0x00202000, 0x10202000, 0x10000000,
			0x10002000, 0x00000008, 0x10200008, 0x00202000,
			0x10202008, 0x00200000, 0x00002008, 0x10000008,
			0x00200000, 0x10002000, 0x10000000, 0x00002008,
			0x10000008, 0x10202008, 0x00202000, 0x10200000,
			0x00202008, 0x10202000, 0x00000000, 0x10200008,
			0x00000008, 0x00002000, 0x10200000, 0x00202008,
			0x00002000, 0x00200008, 0x10002008, 0x00000000,
			0x10202000, 0x10000000, 0x00200008, 0x10002008
		},
		{
			0x00000080, 0x01040080, 0x01040000, 0x21000080,
			0x00040000, 0x00000080, 0x20000000, 0x01040000,
			0x20040080, 0x00040000, 0x01000080, 0x20040080,
			0x21000080, 0x21040000, 0x00040080, 0x20000000,
			0x01000000, 0x20040000, 0x20040000, 0x00000000,
			0x20000080, 0x21040080, 0x21040080, 0x01000080,
			0x21040000, 0x20000080, 0x00000000, 0x21000000,
			0x01040080, 0x01000000, 0x21000000, 0x00040080,
			0x00040000, 0x21000080, 0x00000080, 0x01000000,
			0x20000000, 0x01040000, 0x21000080, 0x20040080,
			0x01000080, 0x20000000, 0x21040000, 0x01040080,
			0x20040080, 0x00000080, 0x01000000, 0x21040000,
			0x21040080, 0x00040080, 0x21000000, 0x21040080,
			0x01040000, 0x00000000, 0x20040000, 0x21000000,
			0x00040080, 0x01000080, 0x20000080, 0x00040000,
			0x00000000, 0x20040000, 0x01040080, 0x20000080
		},
		{
			0x80401000, 0x80001040, 0x80001040, 0x00000040,
			0x00401040, 0x80400040, 0x80400000, 0x80001000,
			0x00000000, 0x00401000, 0x00401000, 0x80401040,
			0x80000040, 0x00000000, 0x00400040, 0x80400000,
			0x80000000, 0x00001000, 0x00400000, 0x80401000,
			0x00000040, 0x00400000, 0x80001000, 0x00001040,
			0x80400040, 0x80000000, 0x00001040, 0x00400040,
			0x00001000, 0x00401040, 0x80401040, 0x80000040,
			0x00400040, 0x80400000, 0x00401000, 0x80401040,
			0x80000040, 0x00000000, 0x00000000, 0x00401000,
			0x00001040, 0x00400040, 0x80400040, 0x80000000,
			0x80401000, 0x80001040, 0x80001040, 0x00000040,
			0x80401040, 0x80000040, 0x80000000, 0x00001000,
			0x80400000, 0x80001000, 0x00401040, 0x80400040,
			0x80001000, 0x00001040, 0x00400000, 0x80401000,
			0x00000040, 0x00400000, 0x00001000, 0x00401040
		},
		{
			0x00000104, 0x04010100, 0x00000000, 0x04010004,
			0x04000100, 0x00000000, 0x00010104, 0x04000100,
			0x00010004, 0x04000004, 0x04000004, 0x00010000,
			0x04010104, 0x00010004, 0x04010000, 0x00000104,
			0x04000000, 0x00000004, 0x04010100, 0x00000100,
			0x00010100, 0x04010000, 0x04010004, 0x00010104,
			0x04000104, 0x00010100, 0x00010000, 0x04000104,
			0x00000004, 0x04010104, 0x00000100, 0x04000000,
			0x04010100, 0x04000000, 0x00010004, 0x00000104,
			0x00010000, 0x04010100, 0x04000100, 0x00000000,
			0x00000100, 0x00010004, 0x04010104, 0x04000100,
			0x04000004, 0x00000100, 0x00000000, 0x04010004,
			0x04000104, 0x00010000, 0x04000000, 0x04010104,
			0x00000004, 0x00010104, 0x00010100, 0x04000004,
			0x04010000, 0x04000104, 0x00000104, 0x04010000,
			0x00010104, 0x00000004, 0x04010004, 0x00010100
		},
		{
			0x40084010, 0x40004000, 0x00004000, 0x00084010,
			0x00080000, 0x00000010, 0x40080010, 0x40004010,
			0x40000010, 0x40084010, 0x40084000, 0x40000000,
			0x40004000, 0x00080000, 0x00000010, 0x40080010,
			0x00084000, 0x00080010, 0x40004010, 0x00000000,
			0x40000000, 0x00004000, 0x00084010, 0x40080000,
			0x00080010, 0x40000010, 0x00000000, 0x00084000,
			0x00004010, 0x40084000, 0x40080000, 0x00004010,
			0x00000000, 0x00084010, 0x40080010, 0x00080000,
			0x40004010, 0x40080000, 0x40084000, 0x00004000,
			0x40080000, 0x40004000, 0x00000010, 0x40084010,
			0x00084010, 0x00000010, 0x00004000, 0x40000000,
			0x00004010, 0x40084000, 0x00080000, 0x40000010,
			0x00080010, 0x40004010, 0x40000010, 0x00080010,
			0x00084000, 0x00000000, 0x40004000, 0x00004010,
			0x40000000, 0x40080010, 0x40084010, 0x00084000
		},
		{
			0x00808200, 0x00000000, 0x00008000, 0x00808202,
			0x00808002, 0x00008202, 0x00000002, 0x00008000,
			0x00000200, 0x00808200, 0x00808202, 0x00000200,
			0x00800202, 0x00808002, 0x00800000, 0x00000002,
			0x00000202, 0x00800200, 0x00800200, 0x00008200,
			0x00008200, 0x00808000, 0x00808000, 0x00800202,
			0x00008002, 0x00800002, 0x00800002, 0x00008002,
			0x00000000, 0x00000202, 0x00008202, 0x00800000,
			0x00008000, 0x00808202, 0x00000002, 0x00808000,
			0x00808200, 0x00800000, 0x00800000, 0x00000200,
			0x00808002, 0x00008000, 0x00008200, 0x00800002,
			0x00000200, 0x00000002, 0x00800202, 0x00008202,
			0x00808202, 0x00008002, 0x00808000, 0x00800202,
			0x00800002, 0x00000202, 0x00008202, 0x00808200,
			0x00000202, 0x00800200, 0x00800200, 0x00000000,
			0x00008002, 0x00008200, 0x00000000, 0x00808002
		}
	};



}



