///////////////////////////////////////////////////////////////////////////////
//                                                                           //
//  Notice to licensees:                                                     //
//                                                                           //
//  This source code is the exclusive, proprietary intellectual property of  //
//  Sharkysoft (sharkysoft.com).  You may view this source code as a         //
//  supplement to other product documentation, but you may not distribute    //
//  it or use it for any other purpose without written consent from          //
//  Sharkysoft.                                                              //
//                                                                           //
//  You are permitted to modify and recompile this source code, but you may  //
//  not remove this notice.  If you add features to or fix errors in this    //
//  code, please consider sharing your changes with Sharkysoft for possible  //
//  incorporation into future releases of the product.  Thanks!              //
//                                                                           //
//  For more information about Sharkysoft products and services, please      //
//  visit Sharkysoft on the web at                                           //
//                                                                           //
//       http://sharkysoft.com/                                              //
//                                                                           //
//  Thank you for using Lava!                                                //
//                                                                           //
///////////////////////////////////////////////////////////////////////////////



package lava.security.des;



/******************************************************************************
DES key.

<p><b>Details:</b>  This class defines a 3DES key, which can be used to initialize a TripleDesCipher.</p>

<p><b>bit ordering</b></p>

<p><strong>Refer to the package summary for important information regarding bit ordering.</strong></p>

@see DesKey
@see TripleDesEngine
@since 1999.08.08
@author Sharky
******************************************************************************/

public final class TripleDesKey
{



	/**********************************************************************
	The first DES key.
	**********************************************************************/

	final DesKey key1;



	/**********************************************************************
	The first DES key.
	**********************************************************************/

	final DesKey key2;



	/**********************************************************************
	The first DES key.
	**********************************************************************/

	final DesKey key3;



	/**********************************************************************
	Specifies all three keys.

	<p><b>Details:</b>  This constructor initializes a new 3DES with the three given single-<wbr>DES keys.</p>

	@param key1 the first key
	@param key2 the second key
	@param key3 the third key
	**********************************************************************/

	public TripleDesKey (DesKey key1, DesKey key2, DesKey key3)
	{
		this . key1 = key1;
		this . key2 = key2;
		this . key3 = key3;
	}



	/**********************************************************************
	Specifies 64-bit keys.

	<p><b>Details:</b>  This constructor initializes a new 3DES key using the three given 64-bit integers.  The first key is index by <var>offset</var>.</p>

	@param keys the 64-bit keys
	@param offset index of first key
	**********************************************************************/

	public TripleDesKey (long[] keys, int offset)
	{
		this
		(
			new DesKey (keys [offset + 0]),
			new DesKey (keys [offset + 1]),
			new DesKey (keys [offset + 2])
		);
	}



	/**********************************************************************
	Specifies 32-bit left and right halves.

	<p><b>Details:</b>  This constructor initializes the key using the 32-bit left and right halves provided.  The left half of the first key is index by <var>offset</var> and the right half is index by <var>offset</var> + 1.</p>

	@param halves the left and right halves
	@param offset index of left half
	**********************************************************************/

	public TripleDesKey (int[] halves, int offset)
	{
		this
		(
			new DesKey (halves, offset    ),
			new DesKey (halves, offset + 1),
			new DesKey (halves, offset + 2)
		);
	}



	/**********************************************************************
	Specifies key as a byte sequence.

	<p><b>Details:</b>  This constructor initializes the key using the given 24 bytes.  The first 8 bytes make up the first key.  The bytes are assembled in big endian order, and the first byte of the first key is indexed by <var>offset</var></p>

	@param bytes the byte array
	@param offset index of first byte
	**********************************************************************/

	public TripleDesKey (byte[] bytes, int offset)
	{
		this
		(
			new DesKey (bytes, offset     ),
			new DesKey (bytes, offset +  8),
			new DesKey (bytes, offset + 16)
		);
	}



	/**********************************************************************
	Tests for odd parity.

	<p><b>Details:</b>  This method determines whether all three of the individual DES keys that make up this 3DES key have odd parity.</p>

	@return true iff parity is odd
	@see DesKey#hasOddParity()
	**********************************************************************/

	public boolean hasOddParity ()
	{
		return
			key1 . hasOddParity ()
		&&	key2 . hasOddParity ()
		&&	key3 . hasOddParity ();
	}



}



